# coding=utf-8
from babel import Locale
from babel.dates import UTC, get_timezone

from allspeak import RequestManager

from .conftest import make_get_request


def test_init():
    rm = RequestManager()
    assert rm.default_locale == Locale('en')
    assert rm.default_timezone == UTC


def test_init_defaults():
    rm = RequestManager(default_locale='es', default_timezone='America/Lima')
    assert rm.default_locale == Locale('es')
    assert rm.default_timezone == get_timezone('America/Lima')


def test_get_locale_default():
    locale = Locale('es')
    rm = RequestManager(default_locale=locale)
    assert rm.get_locale() == locale


def test_get_locale_from_request():
    locale = Locale('es', 'PE')
    default = Locale('en')
    get_request = make_get_request(locale=locale)
    rm = RequestManager(get_request=get_request, default_locale=default)
    assert rm.get_locale() == locale


def test_get_locale_from_request_invalid():
    locale = None
    default = Locale('en')
    get_request = make_get_request(locale=locale)
    rm = RequestManager(get_request=get_request, default_locale=default)
    assert rm.get_locale() == default


def test_get_timezone_default():
    tzinfo = get_timezone('America/Lima')
    rm = RequestManager(default_timezone=tzinfo)
    assert rm.get_timezone() == tzinfo


def test_get_timezone_from_request():
    tzinfo = get_timezone('America/Lima')
    default = UTC
    get_request = make_get_request(tzinfo=tzinfo)
    rm = RequestManager(get_request=get_request, default_timezone=default)
    assert rm.get_timezone() == tzinfo


def test_get_timezone_from_request_invalid():
    tzinfo = None
    default = UTC
    get_request = make_get_request(tzinfo=tzinfo)
    rm = RequestManager(get_request=get_request, default_timezone=default)
    assert rm.get_timezone() == default



# coding=utf-8
import pytest

from django.conf import settings
from django.http import HttpRequest as DjangoRequest
from webob import Request as WebobRequest
from werkzeug.wrappers import Request
from werkzeug.test import EnvironBuilder


settings.configure()


def make_werkzeug_request(env, path):
    return Request(env)


def make_webob_request(env, path):
    return WebobRequest(dict(env))


def make_django_request(env, path):
    r = DjangoRequest()
    r.path = path
    r.META = dict(env)
    return r


def get_test_env(path, **kwargs):
    builder = EnvironBuilder(path=path, **kwargs)
    return builder.get_environ()


def get_test_request(make_req, path='/', **kwargs):
    env = get_test_env(path, **kwargs)
    return make_req(env, path)


def make_get_request(locale='es_PE', tzinfo='America/Lima'):
    """Returns a fake request generator for testing
    """
    _locale = locale
    _tzinfo = tzinfo

    def get_request():
        req = get_test_request(make_werkzeug_request)
        req.locale = _locale
        req.tzinfo = _tzinfo
        return req

    return get_request


@pytest.fixture(params=['werkzeug', 'webob', 'django'])
def make_req(request):
    return globals()['make_{0}_request'.format(request.param)]



# coding=utf-8
from __future__ import print_function

from os.path import join, dirname, abspath

from allspeak import Reader


LOCALES_TEST = abspath(join(dirname(__file__), u'locales'))


def test_reader_repr():
    reader = Reader()
    assert repr(reader) == 'Reader()'


def test_default_loaders():
    reader = Reader()
    assert reader.folderpath
    assert reader.loaders
    assert 'yml' in reader.loaders_ext


def test_extract_locales():
    reader = Reader()
    data = {
        'es-PE': {'foo-es': u'bar'},
        'en': {'foo-en': u'bar'},
    }
    expected = sorted([
        ('es_PE', {'foo-es': u'bar'}),
        ('en', {'foo-en': u'bar'}),
    ])
    assert expected == sorted(reader.extract_locales(data))


def test_load_file():
    reader = Reader()
    filepath = join(LOCALES_TEST, 'en.yml')
    locales = reader.load_file(filepath)
    locale, data = locales[0]
    assert locale == 'en'
    assert data['greeting'] == u'Hello World!'


def test_load_file_multilang():
    reader = Reader()
    filepath = join(LOCALES_TEST, 'multilang.yml')
    locales = reader.load_file(filepath)
    locales = sorted(locales)

    locale, data = locales[0]
    assert locale == 'en'
    assert data['cat'] == u'miaow'

    locale, data = locales[1]
    assert locale == 'es'
    assert data['cat'] == u'miau'


def test_load_translations():
    reader = Reader(LOCALES_TEST)
    data = reader.load_translations()
    assert sorted(data.keys()) == ['en', 'es', 'es_PE']
    assert data['en']['cat'] == 'miaow'
    assert data['en']['foo'] == 'bar'
    assert data['es']['foo'] == 'bares'


def test_load_translations_from_other_folder():
    reader = Reader('.')
    data = reader.load_translations(folderpath=LOCALES_TEST)
    assert sorted(data.keys()) == ['en', 'es', 'es_PE']
    assert data['en']['cat'] == u'miaow'
    assert data['en']['foo'] == u'bar'
    assert data['es']['foo'] == u'bares'

    assert data['es']['accented'] == u'Olé niños y niñas'



# coding=utf-8
from markupsafe import Markup

from . import utils
from ._compat import string_types
from .reader import Reader
from .request_manager import RequestManager
from .utils import locale_to_str


class I18n(RequestManager):
    """Internationalization functions.

    Uses the :class:`Reader` class to load and parse the translations files.

    :param folderpath: path that will be searched for the translations.

    :param markup: overwrite the function used by `translate` to flags HTML
        code as 'safe'. `markupsafe.Markup` is used by default.

    :param get_request: a callable that returns the current request.

    :param default_locale: default locale (as a string or as a
        Babel.Locale instance).

    :param default_timezone: default timezone (as a string or as a
        `datetime.tzinfo` instance).

    :param date_formats: update the defaults date formats.
    """

    def __init__(self, folderpath=utils.LOCALES_FOLDER, markup=Markup, **kwargs):
        self.reader = Reader(folderpath)
        self.markup = markup
        super(I18n, self).__init__(**kwargs)

    def __repr__(self):
        return '{cname}()'.format(
            cname=self.__class__.__name__,
        )

    def load_translations(self, locale=None):
        self.translations = self.reader.load_translations(locale=locale)

    def get_translations_from_locale(self, locale):
        """Return the available translations for a locale: the
        country-specific (is defined) and the one for the language in general.

        :param locale: must be a :class:`babel.core.Locale` instance
        """
        if not self.translations:
            self.load_translations(locale)

        objs = []
        strlocale = locale_to_str(locale)
        trans = self.translations.get(strlocale)
        if trans:
            objs.append(trans)
        if '_' not in strlocale:
            return objs

        strlocale = strlocale.split('_')[0]
        trans = self.translations.get(strlocale)
        if trans:
            objs.append(trans)
        return objs

    def key_lookup(self, locale, key):
        """Return the value of the translation for the given key using the
        current locale. It tries first with the country-specific (eg `en-US`)
        translations (if the locale it's that specific) and then with those
        of the general language (eg. `en`).

        :param locale: must be a :class:`babel.core.Locale` instance
        :param key: a string, the ID of the looked up translation
        """
        translations = self.get_translations_from_locale(locale)
        if not translations:
            # Language not found!
            return None

        for trans in translations:
            for subkey in key.split('.'):
                value = trans.get(subkey)
                if value is None:
                    break
                trans = value
            if value is not None:
                return value
        return None

    def translate(self, key, count=None, locale=None, **kwargs):
        """Get the translation for the given key using the current locale.

        If the value is a dictionary, and `count` is defined, uses the value
        whose key is that number.  If that key doesn't exist, a `'n'` key
        is tried instead.  If that doesn't exits either, an empty string is
        returned.

        The final value is formatted using `kwargs` (and also `count` if
        available) so the format placeholders must be named instead of
        positional.

        If the value isn't a dictionary or a string, is returned as is.

        Examples:

            >>> translate('hello_world')
            'hello %(what)s'
            >>> translate('hello_world', what='world')
            'hello world'
            >>> translate('a_list', what='world')
            ['a', 'b', 'c']

        """
        key = str(key)
        locale = utils.normalize_locale(locale) or self.get_locale()
        value = self.key_lookup(locale, key)
        if value is None:
            return self.markup('<missing:%s>' % (key, ))

        if isinstance(value, dict):
            value = utils.pluralize(value, count)

        if isinstance(value, string_types):
            kwargs.setdefault('count', count)
            value = value % kwargs
            if key.endswith('_html'):
                return self.markup(value)

        return value

    @property
    def lazy_translate(self):
        class LazyWrapper(object):
            def __init__(self_, *args, **kwargs):
                self_.args = args
                self_.kwargs = kwargs

            def __repr__(self_):
                return self.translate(*self_.args, **self_.kwargs)

        return LazyWrapper



# coding=utf-8
from __future__ import print_function

from allspeak import utils
from babel import Locale
from pytz import timezone

from .conftest import (
    make_werkzeug_request, make_webob_request, make_django_request,
    get_test_request,
)


def test_content_negotiation(make_req):
    available_locales = ['en', 'es', 'es-PE']

    headers = [('Accept-Language', 'fr; q=1.0, es; q=0.5, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.negotiate_locale(req, available_locales) == Locale('es')

    headers = [('Accept-Language', 'fr; q=1.0, en-US; q=0.5, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.negotiate_locale(req, available_locales) == Locale('en')

    headers = [('Accept-Language', 'fr; q=1.0, es-PE; q=0.5, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.negotiate_locale(req, available_locales) == Locale('es', 'PE')

    headers = [('Accept-Language', 'fr; q=1.0, es-PE; q=0.5, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.negotiate_locale(req, ['ru']) is None

    headers = [('Accept-Language', 'martian; q=1.0, venus; q=0.5, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.negotiate_locale(req, ['klingon']) is None

    assert utils.negotiate_locale(req, []) is None


def test_get_preferred_locales(make_req):
    headers = [('Accept-Language', 'fr; q=1.0, es; q=0.6, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.get_preferred_locales(req) == ['fr', 'es', 'pt']

    headers = [('Accept-Language', 'fr; q=1.0, en-US; q=0.6, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.get_preferred_locales(req) == ['fr', 'en_US', 'pt']

    headers = [('Accept-Language', 'fr; q=1.0, es-PE; q=0.6, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.get_preferred_locales(req) == ['fr', 'es_PE', 'pt']

    headers = [('Accept-Language', 'fr; q=1.0, es_pe; q=0.6, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.get_preferred_locales(req) == ['fr', 'es_PE', 'pt']

    headers = [('Accept-Language', 'martian; q=1.0, venus; q=0.6, pt; q=0.5')]
    req = get_test_request(make_req, headers=headers)
    assert utils.get_preferred_locales(req) == ['martian', 'venus', 'pt']

    req = get_test_request(make_req)
    assert utils.get_preferred_locales(req) == []


def test_split_locale():
    assert utils.split_locale('en-US') == ('en', 'US')
    assert utils.split_locale('En_us') == ('en', 'US')
    assert utils.split_locale(Locale('en', 'US')) == ('en', 'US')


def test_normalize_locale():
    assert utils.normalize_locale('es') == Locale('es')
    assert utils.normalize_locale('en-US') == Locale('en', 'US')
    assert utils.normalize_locale('en_us') == Locale('en', 'US')
    assert utils.normalize_locale('en-us') == Locale('en', 'US')
    assert utils.normalize_locale('en_US') == Locale('en', 'US')
    assert utils.normalize_locale('En-Us') == Locale('en', 'US')

    assert utils.normalize_locale(Locale('en', 'US')) == Locale('en', 'US')

    assert utils.normalize_locale(('es', )) == Locale('es')
    assert utils.normalize_locale(('en', 'US')) == Locale('en', 'US')
    assert utils.normalize_locale(['en', 'US']) == Locale('en', 'US')
    assert utils.normalize_locale(['EN', 'us']) == Locale('en', 'US')
    assert utils.normalize_locale(['en', 'US', 'Texas']) == Locale('en', 'US')

    assert utils.normalize_locale('klingon') is None
    assert utils.normalize_locale(None) is None
    assert utils.normalize_locale(1) is None


def test_normalize_timezone():
    assert utils.normalize_timezone(timezone('America/Lima')) == timezone('America/Lima')
    assert utils.normalize_timezone('America/Lima') == timezone('America/Lima')
    assert utils.normalize_timezone('Mars') == None


def test_locale_to_str():
    assert utils.locale_to_str(Locale('en', 'US')) == 'en_US'
    assert utils.locale_to_str(Locale('es')) == 'es'
    assert utils.locale_to_str('en_US') == 'en_US'
    assert utils.locale_to_str(Locale('en', 'us')) == 'en_US'


def test_get_werkzeug_preferred_locales():
    headers = [('Accept-Language', 'fr; q=1.0, es; q=0.5, pt; q=0.6')]
    req = get_test_request(make_werkzeug_request, headers=headers)
    langs = utils.get_werkzeug_preferred_locales(req)
    assert langs == ['fr', 'pt', 'es']


def test_get_webob_preferred_locales():
    headers = [('Accept-Language', 'fr; q=1.0, es; q=0.5, pt; q=0.6')]
    req = get_test_request(make_webob_request, headers=headers)
    langs = utils.get_webob_preferred_locales(req)
    assert langs == ['fr', 'pt', 'es']


def test_get_django_preferred_locales():
    headers = [('Accept-Language', 'fr; q=1.0, es; q=0.5, pt; q=0.6')]
    req = get_test_request(make_django_request, headers=headers)
    langs = utils.get_django_preferred_locales(req)
    assert langs == ['fr', 'pt', 'es']


def test_get_request_timezone():
    class FakeRequest(object):
        pass

    request = FakeRequest()

    request.tzinfo = 'America/Lima'
    assert utils.get_request_timezone(request) == timezone('America/Lima')

    request.tzinfo = timezone('America/Lima')
    assert utils.get_request_timezone(request) == timezone('America/Lima')

    request.tzinfo = None
    assert utils.get_request_timezone(request) == None

    request.tzinfo = 'Mars'
    assert utils.get_request_timezone(request) == None

    request.tzinfo = 'Mars'
    assert (
        utils.get_request_timezone(request, default=utils.DEFAULT_TIMEZONE) ==
        utils.DEFAULT_TIMEZONE
    )


def test_get_request_locale():
    class FakeRequest(object):
        pass

    request = FakeRequest()

    request.locale = 'es'
    assert utils.get_request_locale(request) == Locale('es')

    request.locale = 'en-US'
    assert utils.get_request_locale(request) == Locale('en', 'US')

    request.locale = 'en_US'
    assert utils.get_request_locale(request) == Locale('en', 'US')

    request.locale = Locale('en', 'US')
    assert utils.get_request_locale(request) == Locale('en', 'US')

    request.locale = ('es', )
    assert utils.get_request_locale(request) == Locale('es')

    request.locale = ('en', 'US')
    assert utils.get_request_locale(request) == Locale('en', 'US')

    request.locale = 'En-Us'
    assert utils.get_request_locale(request) == Locale('en', 'US')

    request.locale = None
    assert utils.get_request_locale(request) == None

    request.locale = 'klingon'
    assert utils.get_request_locale(request) == None

    request.locale = 'klingon'
    assert (
        utils.get_request_locale(request, default=utils.DEFAULT_LOCALE) ==
        utils.DEFAULT_LOCALE
    )


def test_pluralize():
    d = {
        0: u'No apples',
        1: u'One apple',
        3: u'Few apples',
        'n': u'%(count)s apples',
    }
    assert utils.pluralize(d, 0) == u'No apples'
    assert utils.pluralize(d, 1) == u'One apple'
    assert utils.pluralize(d, 3) == u'Few apples'
    assert utils.pluralize(d, 10) == u'%(count)s apples'

    d = {
        0: u'off',
        'n': u'on'
    }
    assert utils.pluralize(d, 3) == u'on'

    d = {
        0: u'off',
        'n': u'on'
    }
    assert utils.pluralize(d, 0) == u'off'
    assert utils.pluralize(d, None) == u'off'
    assert utils.pluralize({}, 3) == u''



# coding=utf-8
import datetime as dt
from decimal import Decimal

from babel import dates, numbers
from babel.dates import UTC

from . import utils
from .request_manager import RequestManager


class L10n(RequestManager):
    """Localization functions.

    :param get_request: a callable that returns the current request.

    :param default_locale: default locale (as a string or as a
        Babel.Locale instance).

    :param default_timezone: default timezone (as a string or as a
        `datetime.tzinfo` instance).

    :param date_formats: update the defaults date formats.

    """

    def to_user_timezone(self, datetime, tzinfo=None):
        """Convert a datetime object to the user's timezone.  This
        automatically happens on all date formatting unless rebasing is
        disabled.  If you need to convert a `datetime.datetime` object at any
        time to the user's timezone (as returned by `get_timezone` this
        function can be used).

        """
        if datetime.tzinfo is None:
            datetime = datetime.replace(tzinfo=UTC)
        tzinfo = tzinfo or self.get_timezone()
        tzinfo = utils.normalize_timezone(tzinfo)
        return tzinfo.normalize(datetime.astimezone(tzinfo))

    def to_utc(self, datetime, tzinfo=None):
        """Convert a datetime object to UTC and drop tzinfo.
        """
        if datetime.tzinfo is None:
            tzinfo = tzinfo or self.get_timezone()
            tzinfo = utils.normalize_timezone(tzinfo)
            datetime = tzinfo.localize(datetime)
        return datetime.astimezone(UTC).replace(tzinfo=None)

    def _get_format(self, key, format):
        """Gets the date format for that key.
        Helper for the datetime formatting functions.
        """
        if format is None:
            format = self.date_formats.get(key)
        if format in ('short', 'medium', 'full', 'long'):
            dkey = '{key}.{format}'.format(key=key, format=format)
            rv = self.date_formats.get(dkey)
            if rv is not None:
                format = rv
        return format

    def _date_format(self, formatter, obj, format, rebase,
                     locale=None, tzinfo=None, **extra):
        locale = utils.normalize_locale(locale) or self.get_locale()
        extra = {}
        if formatter is not dates.format_date and rebase:
            tzinfo = tzinfo or self.get_timezone()
            extra['tzinfo'] = utils.normalize_timezone(tzinfo)
        return formatter(obj, format, locale=locale, **extra)

    def format(self, value, *args, **kwargs):
        """Return a formatted `value` according to the detected type and
        given parameters.

        It doesn't know anything about currency, percent or
        scientific formats, so use the other methods for those cases.
        """
        locale = kwargs.pop('locale', None)
        tzinfo = kwargs.pop('tzinfo', None)

        if isinstance(value, dt.date):
            if isinstance(value, dt.datetime):
                return self.format_datetime(
                    value, locale=locale, tzinfo=tzinfo, *args, **kwargs)
            else:
                return self.format_date(
                    value, locale=locale, tzinfo=tzinfo, *args, **kwargs)

        if isinstance(value, int):
            return self.format_number(value, locale=locale, *args, **kwargs)
        if isinstance(value, (float, Decimal)):
            return self.format_decimal(value, locale=locale, *args, **kwargs)

        if isinstance(value, dt.time):
            return self.format_time(value, locale=locale, tzinfo=tzinfo,
                                    *args, **kwargs)
        if isinstance(value, dt.timedelta):
            return self.format_timedelta(value, locale=locale, *args, **kwargs)

        return value

    def format_datetime(self, datetime=None, format=None, rebase=True,
                        locale=None, tzinfo=None, **kwargs):
        """Return a datetime formatted according to the given pattern.

        :param datetime: A `datetime.datetime` object.
            If no object is passed, the current datetime is assumed.

        :param format: The format parameter can either be `'short'`, `'medium'`,
            `'long'` or `'full'` (in which cause the language's default for
            that setting is used) or a format string as
            `documented by Babel <http://babel.pocoo.org/docs/dates/#date-fields>`_.

        :param rebase: Convert the given `date` to the users's timezone (as returned by
            :meth:`to_user_timezone`)
            By default rebasing happens.

        :param locale: Overwrite the global locale.
        :param tzinfo: Overwrite the global timezone.

        """
        format = self._get_format('datetime', format)
        return self._date_format(
            dates.format_datetime, datetime, format, rebase,
            locale=locale, tzinfo=tzinfo, **kwargs
        )

    def format_date(self, date=None, format=None, rebase=True,
                    locale=None, tzinfo=None, **kwargs):
        """Return a date formatted according to the given pattern.

        :param date: A `datetime.datetime` or `datetime.date` object.
            If no object is passed, the current datetime is assumed.

        :param format: The format parameter can either be `'short'`, `'medium'`,
            `'long'` or `'full'` (in which cause the language's default for
            that setting is used) or a format string as
            `documented by Babel <http://babel.pocoo.org/docs/dates/#date-fields>`_.

        :param rebase: Convert the given `date` to the users's timezone (as returned by
            :meth:`to_user_timezone`)
            By default rebasing happens.

        :param locale: Overwrite the global locale.
        :param tzinfo: Overwrite the global timezone.

        """
        if rebase and isinstance(date, dt.datetime):
            date = self.to_user_timezone(date, tzinfo=tzinfo)
        format = self._get_format('date', format)
        return self._date_format(
            dates.format_date, date, format, rebase,
            locale=locale, tzinfo=tzinfo, **kwargs
        )

    def format_time(self, time=None, format=None, rebase=True,
                    locale=None, tzinfo=None, **kwargs):
        """Return a time formatted according to the given pattern.

        :param time: A `time` or `datetime` object.
            If no object is passed, the current time is assumed.

        :param format: The format parameter can either be `'short'`, `'medium'`,
            `'long'` or `'full'` (in which cause the language's default for
            that setting is used) or a format string as
            `documented by Babel <http://babel.pocoo.org/docs/dates/#time-fields>`_.

        :param rebase: Convert the given `time` to the users's timezone (as returned by
            :meth:`to_user_timezone`).
            By default rebasing happens.

        :param locale: Overwrite the global locale.
        :param tzinfo: Overwrite the global timezone.

        """
        format = self._get_format('time', format)
        return self._date_format(
            dates.format_time, time, format, rebase,
            locale=locale, tzinfo=tzinfo, **kwargs
        )

    def format_timedelta(self, datetime_or_timedelta, granularity='second',
                         threshold=0.85, add_direction=False, format='medium',
                         locale=None, **kwargs):
        """Format the elapsed time from the given date to now or the given
        timedelta as documented in :func:`babel.dates.format_timedelta`.

        :param delta: a timedelta object representing the time difference to
            format, or the delta in seconds as an int value.

        :param granularity: determines the smallest unit that should be
            displayed, the value can be one of “year”, “month”, “week”, “day”,
            “hour”, “minute” or “second”.

        :param threshold: factor that determines at which point the
            presentation switches to the next higher unit.

        :param add_direction: if this flag is set to True the
            return value will include directional information. For instance a
            positive timedelta will include the information about it being in
            the future, a negative will be information about the value being in
            the past.

        :param format: the format (currently only “medium” and “short” are supported)
        :param locale: Overwrite the global locale.

        """
        locale = utils.normalize_locale(locale) or self.get_locale()
        if isinstance(datetime_or_timedelta, dt.datetime):
            datetime_or_timedelta = dt.datetime.utcnow() - datetime_or_timedelta
        return dates.format_timedelta(
            datetime_or_timedelta,
            granularity=granularity, threshold=threshold,
            add_direction=add_direction, locale=locale,
            **kwargs
        )

    def format_number(self, number, locale=None, **kwargs):
        """Return the given number formatted for the locale in the
        current request.

        :param number: the number to format
        :param locale: Overwrite the global locale.

        """
        locale = utils.normalize_locale(locale) or self.get_locale()
        return numbers.format_number(number, locale=locale, **kwargs)

    def format_decimal(self, number, format=None, locale=None, **kwargs):
        """Return the given decimal number formatted for the locale in the
        current request.

        :param number: the number to format
        :param format: the format to use as
            `documented by Babel <http://babel.pocoo.org/docs/numbers/#pattern-syntax>`_.
        :param locale: Overwrite the global locale.

        """
        locale = utils.normalize_locale(locale) or self.get_locale()
        return numbers.format_decimal(number, format=format, locale=locale, **kwargs)

    def format_currency(self, number, currency, format=None, locale=None, **kwargs):
        """Return the given number formatted for the locale in the
        current request.

        :param number: the number to format
        :param currency: the currency code
        :param format: the format to use as
            `documented by Babel <http://babel.pocoo.org/docs/numbers/#pattern-syntax>`_.
        :param locale: Overwrite the global locale.

        """
        locale = utils.normalize_locale(locale) or self.get_locale()
        return numbers.format_currency(
            number, currency, format=format, locale=locale, **kwargs)

    def format_percent(self, number, format=None, locale=None, **kwargs):
        """Return a percent value formatted for the locale in the
        current request.

        :param number: the number to format
        :param format: the format to use as
            `documented by Babel <http://babel.pocoo.org/docs/numbers/#pattern-syntax>`_.
        :param locale: Overwrite the global locale.

        """
        locale = utils.normalize_locale(locale) or self.get_locale()
        return numbers.format_percent(number, format=format, locale=locale, **kwargs)

    def format_scientific(self, number, format=None, locale=None, **kwargs):
        """Return value formatted in scientific notation for the locale in
        the current request.

        :param number: the number to format
        :param format: the format to use as
            `documented by Babel <http://babel.pocoo.org/docs/numbers/#pattern-syntax>`_.
        :param locale: Overwrite the global locale.

        """
        locale = utils.normalize_locale(locale) or self.get_locale()
        return numbers.format_scientific(number, format=format, locale=locale, **kwargs)



# coding=utf-8
from babel import Locale
from babel.dates import get_timezone

from . import utils
from .utils import DEFAULT_LOCALE, DEFAULT_TIMEZONE, DEFAULT_DATE_FORMATS


class RequestManager(object):
    """
    A base class with methods for getting the locale and timezone
    from the request object.

    :param get_request: a callable that returns the current request.

    :param default_locale: default locale (as a string or as a
        Babel.Locale instance).

    :param default_timezone: default timezone (as a string or as a
        `datetime.tzinfo` instance).

    :param date_formats: update the defaults date formats.

    """

    def __init__(self, get_request=None, default_locale=DEFAULT_LOCALE,
                 default_timezone=DEFAULT_TIMEZONE, date_formats=None):
        self.get_request = get_request
        self.set_defaults(default_locale, default_timezone)
        self.date_formats = DEFAULT_DATE_FORMATS.copy()
        if date_formats:
            self.date_formats.update(date_formats)
        self.translations = {}

    def __repr__(self):
        return '{cname}(default_locale={default_locale}, default_timezone={default_timezone})'.format(
            cname=self.__class__.__name__,
            default_locale=self.default_locale,
            default_timezone=self.default_timezone,
        )

    def set_defaults(self, default_locale, default_timezone):
        """Set the default locale from the configuration as an instance of
        :class:`babel.core.Locale` and the default timezone as a
        `datetime.tzinfo`.

        """
        self.default_locale = (
            utils.normalize_locale(default_locale) or
            Locale(DEFAULT_LOCALE)
        )
        self.default_timezone = (
            utils.normalize_timezone(default_timezone) or
            get_timezone(DEFAULT_TIMEZONE)
        )

    def get_locale(self):
        """Returns the locale that should be used for this request as
        an instance of :class:`babel.core.Locale`.
        This returns the default locale if used outside of a request.

        """
        request = self.get_request and self.get_request()
        if not request:
            return self.default_locale
        return utils.get_request_locale(request, self.default_locale)

    def get_timezone(self):
        """Returns the timezone that should be used for this request as
        `datetime.tzinfo` instance.  This returns the default timezone if used
        outside of a request or if no timezone was defined.

        """
        request = self.get_request and self.get_request()
        if not request:
            return self.default_timezone
        return utils.get_request_timezone(request, self.default_timezone)



# coding=utf-8

from allspeak import Allspeak, L10n, I18n


def test_allspeak_repr():
    speak = Allspeak()
    assert repr(speak) == 'Allspeak()'


def test_allspeak_has_l10n():
    speak = Allspeak()
    l10n = L10n()
    for key in l10n.__dict__:
        assert hasattr(speak, key)


def test_allspeak_has_i18n():
    speak = Allspeak()
    i18n = I18n()
    for key in i18n.__dict__:
        assert hasattr(speak, key)



# coding=utf-8
"""
========================
AllSpeak
========================

AllSpeak is a pythonic (yet ironically inspired by Rails) i18n/l10n solution
for humans. It's flexible, easy to use and —unlike gettext— independent of
any external compilation tool.

How can the translator of your multi-language web application update a text?
Compiling `.po` files for a web app, really? How the Rails community solved
that problem? Translations in `yaml` or `json` files. With Python it should
be that simple. **Now it is**.

And the files used by Allspeak are compatible with those of Rails, so you can
use any third-party service already compatible with them
(for example, `Transifex <https://www.transifex.com/>`_).

Powered by the awesome Babel and pytz libraries for the l10n part.

What's in a name?
----------------------------------------------

    "When Thor speaks with the All-Speak anyone who hears him will hear him
    speak their native language in their hearts" ——(from Thor's wiki page)

:copyright: `Juan-Pablo Scaletti <http://jpscaletti.com>`_.
:license: MIT, see LICENSE for more details.

"""
# import os
# Workaround for a possible (?) OSX and/or Windows bug.
# if os.environ.get('LC_CTYPE', '').lower() == 'utf-8':
#     os.environ['LC_CTYPE'] = 'en_US.utf-8'

from .allspeak import Allspeak  # noqa
from .i18n import I18n  # noqa
from .l10n import L10n  # noqa
from .reader import Reader  # noqa
from .request_manager import RequestManager  # noqa
from .utils import *  # noqa

__version__ = '0.6'



# coding=utf-8
import datetime
from babel import Locale, UnknownLocaleError
from babel.dates import get_timezone, UTC

from ._compat import string_types


__all__ = [
    'LOCALES_FOLDER', 'DEFAULT_LOCALE', 'DEFAULT_TIMEZONE',
    'DEFAULT_DATE_FORMATS', 'normalize_locale', 'normalize_timezone',
    'get_werkzeug_preferred_locales', 'get_webob_preferred_locales',
    'get_django_preferred_locales', 'get_preferred_locales', 'negotiate_locale',
    'get_request_timezone', 'get_request_locale', 'pluralize',
]

LOCALES_FOLDER = 'locales'

DEFAULT_LOCALE = 'en'
DEFAULT_TIMEZONE = UTC

DEFAULT_DATE_FORMATS = {
    'time': 'medium',
    'date': 'medium',
    'datetime': 'medium',
}


def split_locale(locale):
    """Returns a tuple (language, TERRITORY) or just (language, )
    from a a `~Babel.Locale` instance or a string like `en-US` or `en_US`.
    """
    if isinstance(locale, Locale):
        tloc = [locale.language.lower()]
        if locale.territory:
            tloc.append(locale.territory.upper())
        return tuple(tloc)

    if isinstance(locale, string_types):
        locale = locale.replace('-', '_').lower()
        tloc = locale.split('_')
        if len(tloc) > 1:
            tloc[-1] = tloc[-1].upper()
        return tuple(tloc)

    return locale


def normalize_locale(locale):
    if not locale:
        return
    if isinstance(locale, Locale):
        return locale

    locale = split_locale(locale)

    if isinstance(locale, (tuple, list)):
        try:
            if len(locale) == 1:
                return Locale(locale[0].lower())
            else:
                return Locale(locale[0].lower(), locale[1].upper())
        except UnknownLocaleError:
            return None
    return None


def normalize_timezone(tzinfo):
    if not tzinfo:
        return
    if isinstance(tzinfo, datetime.tzinfo):
        return tzinfo
    try:
        return get_timezone(tzinfo)
    except LookupError:
        return


def locale_to_str(locale):
    if not isinstance(locale, Locale):
        return locale
    strlocale = locale.language.lower()
    if locale.territory:
        strlocale += '_' + locale.territory.upper()
    return strlocale


def get_werkzeug_preferred_locales(request):
    """Return a list of preferred languages from a `werkzeug.wrappers.Request`
    instance.

    """
    languages = getattr(request, 'accept_languages', None)
    if languages:
        return [
            '_'.join(split_locale(l))
            for l in languages.values()
        ]


def get_webob_preferred_locales(request):
    """Return a list of preferred languages from a `webob.Request` instance.

    """
    languages = getattr(request, 'accept_language', None)
    if languages:
        return [
            '_'.join(split_locale(l))
            for l in languages
        ]


def get_django_preferred_locales(request):
    """Take a `django.HttpRequest` instance and return a list of preferred
    languages from the headers.

    """
    meta = getattr(request, 'META', None)
    if not meta:
        return None
    header = request.META.get('HTTP_ACCEPT_LANGUAGE')
    if header:
        languages = [l.strip().split(';')[::-1] for l in header.split(',')]
        languages = sorted(languages)[::-1]
        return [
            '_'.join(split_locale(l[1].strip()))
            for l in languages
        ]


def get_preferred_locales(request):
    """Extract from the request a list of preferred strlocales.
    """
    return (
        get_werkzeug_preferred_locales(request) or
        get_webob_preferred_locales(request) or
        get_django_preferred_locales(request) or
        []
    )


def negotiate_locale(request, available_locales):
    """From the available locales, negotiate the most adequate for the
    client, based on the "accept language" header.
    """
    if not available_locales:
        return None
    preferred = get_preferred_locales(request)
    if preferred:
        preferred = map(
            lambda l: l.replace('-', '_').lower(),
            preferred
        )
        available_locales = map(
            lambda l: l.replace('-', '_').lower(),
            available_locales
        )
        # To ensure a consistent matching, Babel algorithm is used.
        return Locale.negotiate(preferred, available_locales, sep='_')


def get_request_timezone(request, default=None):
    """Returns the timezone that should be used for this request as a
    `datetime.tzinfo` instance.

    Tries the following in order:

    - an attribute called `'tzinfo'`
    - a GET argument called `'tzinfo'`
    - the provided default timezone

    """
    tzinfo = (
        getattr(request, 'tzinfo', None) or
        getattr(request, 'args', getattr(
                request, 'GET', {})).get('tzinfo')
    )
    request.tzinfo = normalize_timezone(tzinfo) or default
    return request.tzinfo


def get_request_locale(request, default=None):
    """Returns the locale that should be used for this request as a
    `babel.Locale` instance.

    Tries the following in order:

    - an request attribute called `'locale'`
    - a GET argument called `'locale'`
    - the default locale

    """
    locale = (
        getattr(request, 'locale', None) or
        getattr(request, 'args', getattr(request, 'GET', {})).get('locale')
    )
    request.locale = normalize_locale(locale) or default
    return request.locale


def pluralize(dic, count):
    """Takes a dictionary and a number and return the value whose key in
    the dictionary is that number.  If that key doesn't exist, a `'n'` key
    is tried instead.  If that doesn't exits either, an empty string is
    returned.  Examples:

    >>> dic = {
        0: u'No apples',
        1: u'One apple',
        3: u'Few apples',
        'n': u'%(count)s apples',
        }
    >>> pluralize(dic, 0)
    'No apples'
    >>> pluralize(dic, 1)
    'One apple'
    >>> pluralize(dic, 3)
    'Few apples'
    >>> pluralize(dic, 10)
    '%(count)s apples'
    >>> pluralize({0: 'off', 'n': 'on'}, 3)
    'on'
    >>> pluralize({0: 'off', 'n': 'on'}, 0)
    'off'
    >>> pluralize({}, 3)
    ''

    """
    if count is None:
        count = 0
    scount = str(count)
    return dic.get(count, dic.get(scount, dic.get('n', u'')))
